/**********************************************************************
$Id: md5sum.c,v 1.2 2004/07/31 07:53:01 bu Exp $

Copyright 2004, Bulent Yilmaz
for Microbrew Software

Part of Microbrew md5sum/sha1sum
GNU textutils md5sum replacement

http://www.microbrew.org/tools/md5sha1sum.html

Microbrew md5sum/sha1sum is free software; you can redistribute it and/or 
modify it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

MicroSieve is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with MicroSieve; if not, write to the Free Software Foundation, Inc.,
59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

Special exemption from the GPL is given solely for the purpose of linking 
with OpenSSL which is distributed under the BSD license.  This is the only
exemption from the GPL which is given and all parts of the GPL are still
in effect including those parts which are similar to the exemption above.

**********************************************************************/

#include <openssl/md5.h>
#include <openssl/sha.h>

#include "parseopts.h"

char progver[] = "0.1.0";

void do_md5sum(FILE *mfile, char *rptr) {
    char rdata[BUFSIZ];
    int rsize;
    MD5_CTX c;
    
    MD5_Init(&c);
    while(!feof(mfile)) {
        rsize = fread(rdata, sizeof(char), BUFSIZ, mfile);
        if(rsize > 0) {
            MD5_Update(&c, rdata, rsize);
        }
    }

    MD5_Final(rptr, &c);
}

void do_sha1sum(FILE *mfile, char *rptr) {
    char rdata[BUFSIZ];
    int rsize;
    SHA_CTX c;
    
    SHA1_Init(&c);
    while(!feof(mfile)) {
        rsize = fread(rdata, sizeof(char), BUFSIZ, mfile);
        if(rsize > 0) {
            SHA1_Update(&c, rdata, rsize);
        }
    }

    SHA1_Final(rptr, &c);
}

int main(int argc, char *argv[]) {
    struct opts_t *opts;
    int i,j;
    FILE* tfile;
    unsigned char sumdata[SHA_DIGEST_LENGTH];
    char psum[41];
    int fails = 0;
    int hashsize;

    opts = parseopts(argc, argv);
    if(opts == NULL) {
        return -1;
    }

    if(opts->filect <= 0) {
        opts->filenames = malloc(sizeof(char *));
        if(opts->filenames == NULL) {
            perror("main: malloc opts->filenames");
            return -1;
        }

        opts->filenames[0] = "-";
        opts->filect = 1;
    }

    for(i = 0; i < opts->filect; i++) {
        if(!strcmp(opts->filenames[i], "-")) {
            tfile = stdin;
        } else {
            if(opts->fmode == FMODE_BIN) 
                tfile = fopen(opts->filenames[i], "rb");
            else
                tfile = fopen(opts->filenames[i], "r");
        }

        if(tfile == NULL) {
            fprintf(stderr, "Could not open file '%s': ", 
                    opts->filenames[i]);

            perror("");
        } else {
            if(strstr(argv[0], "sha1sum")) {
                do_sha1sum(tfile, sumdata);
                for(j = 0; j < SHA_DIGEST_LENGTH; j++)
                    sprintf(&psum[(j*2)], "%02x", (int)sumdata[j]);
            } else {
                do_md5sum(tfile, sumdata);
                for(j = 0; j < MD5_DIGEST_LENGTH; j++)
                    sprintf(&psum[(j*2)], "%02x", (int)sumdata[j]);
            }

            psum[40] = '\0';

            if(opts->pmode == PMODE_VERIFY) {
                /* Be nice an offer a warning if hash side appears to be 
                   wrong */

                hashsize = strlen(opts->hashes[i]);
                if(strstr(argv[0], "sha1sum") && (hashsize == 32))
                    fputs("Warning: Hash size is wrong, maybe you want to use 'md5sum'\n", stderr);
                else if(!strstr(argv[0], "sha1sum") && hashsize == 40)
                    fputs("Warning: Hash size is wrong, maybe you want to use 'sha1sum'\n", stderr);

                if(opts->vmode == VMODE_SILENT) {
                    if(strcmp(opts->hashes[i], psum)) fails++;
                } else {
                    printf("%s: ", opts->filenames[i]);
                    if(!strcmp(opts->hashes[i], psum)) puts("OK");
                    else {
                        puts("FAILED");
                        fails++;
                    }
                }
            } else {
                if(opts->fmode == FMODE_BIN) 
                    printf("%s *%s\n", psum, opts->filenames[i]);
                else
                    printf("%s  %s\n", psum, opts->filenames[i]);
            }
        }

        if(tfile != stdin) fclose(tfile);
    } 

    if(opts->pmode == PMODE_VERIFY) {
        if(fails > 0) {
            if(opts->vmode != VMODE_SILENT) {
                printf("%s: WARNING: %d of %d computed checksums did NOT match\n", argv[0], fails, opts->filect);
            }
            return 1;
        }
    }

    return 0;
}
